package jalview.ws.slivkaws;

import jalview.gui.AlignFrame;
import jalview.ws.WSMenuEntryProviderI;
import jalview.ws.api.ServiceWithParameters;
import jalview.ws.jws2.MsaWSClient;
import jalview.ws.jws2.PreferredServiceRegistry;
import jalview.ws.jws2.SequenceAnnotationWSClient;

import java.beans.PropertyChangeSupport;
import java.io.IOError;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JMenu;

import uk.ac.dundee.compbio.slivkaclient.SlivkaClient;
import uk.ac.dundee.compbio.slivkaclient.SlivkaService;

public class SlivkaWSDiscoverer implements Runnable, WSMenuEntryProviderI
{
  private static SlivkaWSDiscoverer instance = null;

  private SlivkaClient slivkaClient;

  private SlivkaWSDiscoverer()
  {
    try
    {
      slivkaClient = new SlivkaClient("localhost", 4954);
    } catch (URISyntaxException e)
    {
      throw new RuntimeException(e);
    }
  }

  public static SlivkaWSDiscoverer getInstance()
  {
    if (instance == null)
    {
      instance = new SlivkaWSDiscoverer();
    }
    return instance;
  }

  private void addMsaServiceClient(JMenu alignmentMenu,
          SlivkaService service, AlignFrame alignFrame)
  {
    SlivkaMsaServiceInstance instance = new SlivkaMsaServiceInstance(
            slivkaClient, service);
    MsaWSClient client = new MsaWSClient();
    client.attachWSMenuEntry(alignmentMenu, instance, alignFrame);
    alignmentMenu.addSeparator();
  }

  private void addDisorderServiceClient(JMenu disorderMenu,
          SlivkaService service, AlignFrame alignFrame)
  {
    SlivkaAnnotationServiceInstance serviceInstance = new SlivkaAnnotationServiceInstance(
            slivkaClient, service, false);

    SequenceAnnotationWSClient client = new SequenceAnnotationWSClient();
    client.attachWSMenuEntry(disorderMenu, serviceInstance, alignFrame);
    disorderMenu.addSeparator();
  }

  private void addConservationServiceClient(JMenu conservationMenu,
          SlivkaService service, AlignFrame alignFrame)
  {
    SlivkaAnnotationServiceInstance serviceInstance = new SlivkaAnnotationServiceInstance(
            slivkaClient, service, true);
    SequenceAnnotationWSClient client = new SequenceAnnotationWSClient();
    client.attachWSMenuEntry(conservationMenu, serviceInstance, alignFrame);
    conservationMenu.addSeparator();
  }

  private PropertyChangeSupport changeSupport = new PropertyChangeSupport(
          this);

  @Override
  public void attachWSMenuEntry(JMenu wsmenu, final AlignFrame alignFrame)
  {
    JMenu slivkaMenu = new JMenu("Slivka");
    wsmenu.add(slivkaMenu);

    JMenu alignmentMenu = new JMenu("Sequence Alignment");
    slivkaMenu.add(alignmentMenu);
    JMenu disorderMenu = new JMenu("Protein sequence analysis");
    slivkaMenu.add(disorderMenu);
    JMenu conservationMenu = new JMenu("Conservation");
    slivkaMenu.add(conservationMenu);
    try
    {
      List<ServiceWithParameters> instances = new ArrayList<>();
      for (SlivkaService service : slivkaClient.getServices())
      {
        for (String classifier : service.classifiers)
        {
          if (classifier.contains("Multiple sequence alignment"))
          {
            instances.add(
                    new SlivkaMsaServiceInstance(slivkaClient, service));
            // addMsaServiceClient(alignmentMenu, service, alignFrame);
          }
          else if (classifier.contains("Protein sequence analysis"))
          {
            instances.add(new SlivkaAnnotationServiceInstance(slivkaClient,
                    service, false));
            // addDisorderServiceClient(disorderMenu, service, alignFrame);
          }
          else if (classifier
                  .contains("Sequence alignment analysis (conservation)"))
          {
            instances.add(new SlivkaAnnotationServiceInstance(slivkaClient,
                    service, true));
            // addConservationServiceClient(conservationMenu, service,
            // alignFrame);
          }

        }
      }
      PreferredServiceRegistry.getRegistry().populateWSMenuEntry(instances,
              changeSupport, slivkaMenu, alignFrame, null);
    } catch (IOException e)
    {
      throw new IOError(e);
    }
  }

  @Override
  public void run()
  {

  }
}
